import {ISpotlightOptions} from "../utils/options/ISpotlightOptions";
import {IElementInformation} from "../utils/IElementInformation";
const scrollIntoView = require('scroll-into-view');

export default class Actor {
    private readonly root: HTMLElement;
    private readonly name: string;
    private readonly _options: ISpotlightOptions;

    constructor (root: HTMLElement, options?: ISpotlightOptions) {
        this.name = `${root.tagName}-${root.id}-${root.className}`.toLocaleLowerCase();
        this.root = root;
        this._options = options;
    }


    get options(): ISpotlightOptions {
        return this._options;
    }

    public getInformation (): IElementInformation {
        const rect = this.root.getBoundingClientRect();
        const style = getComputedStyle(this.root);
        return {
            borderRadius: {
                topLeft: style.borderTopLeftRadius,
                topRight: style.borderTopRightRadius,
                bottomLeft: style.borderBottomLeftRadius,
                bottomRight: style.borderBottomRightRadius
            },
            position: {
                x: rect.x -2 + window.scrollX,
                y: rect.y + window.scrollY,
                width: rect.width - 2,
                height: rect.height
            }
        };
    }

    private _isOutOfViewport (): boolean {
        const rect = this.root.getBoundingClientRect();

        // checking if all sides are in
        const out = {
            top: rect.top < 0,
            left: rect.left < 0,
            bottom: rect.bottom > (window.innerHeight || document.documentElement.clientHeight),
            right: rect.right > (window.innerWidth || document.documentElement.clientWidth)
        };

        return out.top || out.left || out.bottom || out.right;
    }

    // https://www.npmjs.com/package/scroll-into-view
    public scrollIntoView (options: ISpotlightOptions) {
        if (this._isOutOfViewport() || options.centerActorOnFocus) {
            scrollIntoView(this.root, {
                time: options.delay
            });
        }
    }

    public toString(): string {
        return `Actor[${this.name}]`;
    }
}
